/************************************************************************************************
 Author        : ADI - Apps            www.analog.com

 Date          : July 2009

 File          : SPI_Master.c

 Hardware      : ADuC7122

 Description   : This SPI Master example shows how to initialise the SPI peripheral in Master mode
 			This simple example transmits 6 bytes to a Slave after a reset.
			When an XIRQ0 interrupt occurs, the Master will then read back 6 bytes from the Slave.
*************************************************************************************************/
// Bit Definitions
#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80
#define BIT8  0x100
#define BIT9  0x200
#define BIT10 0x400
#define BIT11 0x800
#define BIT12 0x1000
#define BIT13 0x2000
#define BIT14 0x4000
#define BIT15 0x8000
#define BIT16 0x10000
#define BIT17 0x20000
#define BIT19 0x80000
#define BIT20 0x100000
#define BIT24 0x1000000
#define BIT25 0x2000000
#define BIT28 0x10000000 

#include <Analogdevices\ioaduc7122.h>
#include <intrinsics.h>

// Function Definitions
unsigned char szTxData[] = {0x1, 0x2, 0x10, 0x20, 0x40, 0x80};	// Array to send to Slave
unsigned char ucTxCount = 0;                // Array index variable for szTxData[]
unsigned char szRxData[6];	            // Array for reading Data from Slave
unsigned char ucRxCount = 0;                // Array index variable for szRxData[]
unsigned char ucWaitingForXIRQ0 = 1;        // Flag to begin reading from Slave - controlled by XIRQ0
unsigned char ucDummyRd = 0;
int main(void)
{
     	POWKEY1 = 0x01;		            // Configure CPU Clock for 41.78MHz, CD=0
 	POWCON = 0x00;
 	POWKEY2 = 0xF4; 

	// Configure P0.2, P0.3,P0.4 and P0.5 for SPI mode
	GP0CON = BIT8 + BIT12 + BIT16 + BIT20;  // Select SPI alternative function 

	SPICON = BIT0 + BIT1                // Enable SPI + Enable SPI Master
		   + BIT6                   // Clk pulses high at start of each bit + Initiate transfer on write to Tx FIFO
		   + BIT11 ;	            // Continuous transfer mode + Enable IRQ on 1 byte transfer
	SPIDIV  =/*0xC7;*/ 0x31;	    // Select 101kHz clock rate
	IRQCONE = BIT1;			    // Rising edge of XIRQ0
	IRQEN = BIT14 + BIT19; 		    // Enable SPI + XIRQ0 interrupt
        IRQCONE=BIT0;//+BIT1;               // External IRQ0 triggers on falling edge
        __enable_interrupt();
        
	// Begin Master Transmit sequence
	ucTxCount = 0;
	SPITX = szTxData[ucTxCount++];
	
	ucWaitingForXIRQ0 = 1;
	ucRxCount = 0;
		while (1)
	{
          if (ucWaitingForXIRQ0 == 0)	    // Wait for XIRQ0 to trigger - cleared in XIRQ0 ISR
		{
		    ucWaitingForXIRQ0 = 1;
		    ucRxCount = 0;
		    SPICON |= BIT12;        // Flush Rx FIFO
		    SPICON &= ~BIT12; 
		    SPICON &= ~BIT6;        // Generate Interrupt on receiving data
                    SPICON = BIT0 + BIT1 +BIT3 + BIT11;// Enable SPI + Enable SPI Master
		    // Clk pulses high at start of each bit + Initiate transfer on write to Tx FIFO
                    szRxData[0]=SPIRX;
		}
	}
}

__irq void IRQ_Handler(void)
{
	unsigned long IRQSTATUS = 0;
	unsigned int SPIMSTATUS = 0;

	IRQSTATUS = IRQSTA;	                // Read off IRQSTA register
	if ((IRQSTATUS & BIT19) == BIT19)	//XIRQ0 interrupt source
	{
		ucWaitingForXIRQ0 = 0;	        // Enable Read request from Slave.
		IRQCLR = BIT19;
	}

	if ((IRQSTATUS & BIT14) == BIT14)	//If SPI Master interrupt source
	{
	   SPIMSTATUS = SPISTA;
	   if ((SPIMSTATUS & BIT4) == BIT4)     // SPI Tx underflow
	   {
	   }
	   if ((SPIMSTATUS & BIT5) == BIT5)     // If SPI Master Tx IRQ
	   {
		if ( ucTxCount < 6)	        // Have max 6 bytes been sent?
		SPITX = szTxData[ucTxCount++];	// Load Tx buffer
	   }
	   if ((SPIMSTATUS & BIT6) == BIT6)     // If SPI Master Rx IRQ
	   {
		if (ucRxCount < 6)              // Have max 6 bytes been received?
	        {
		  szRxData[ucRxCount] = SPIRX;  // Read Rx buffer
		  ucRxCount++;
		}
	   }
	}
}


